/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/


package com.googlepages.dronten.jtagger;

import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.gui.MainWindow;
import com.googlepages.dronten.jtagger.gui.dlg.MessageDialog;
import com.googlepages.dronten.jtagger.gui.dlg.setup.BaseSetup;
import com.googlepages.dronten.jtagger.gui.dlg.setup.GeneralSetup;
import com.googlepages.dronten.jtagger.gui.dlg.setup.MP3Setup;
import com.googlepages.dronten.jtagger.gui.dlg.setup.SetupDialog;
import com.googlepages.dronten.jtagger.thread.AudioThread;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Pref;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;
import java.util.Properties;


/**
 * JTagger is the main application object.<br>
 * Start app here and use JTagger for global calls.<br>
 * JTagger manage the menu bar and creates a MainWindow which owns LeftPanel browser and JTabbedPane which is the container of all child views.<br>
 * Every tab view has an option panel amd a table showing tracks.<br>
 * <br>
 * All child views has the same tracks an selectin a single track in one view will select same track in all other views.<br>
 * JTagger.selectRow syncs all views so the same track is selected.<br>
 * JTagger.fireTableDataChanged2 syncs all data changes for all the views.<br>
 */
public class JTagger extends JFrame implements ActionListener {
    private static final long   serialVersionUID = 1;
    public static ImageIcon     NO_IMAGE = null;
    private static JTagger      JTAGGER = null;

    private int                 aSelectedRow = 0;
    private Cursor              aDefaultCursor = null;
    private Cursor              aWaitCursor = null;
    private MainWindow          aMainWindow = null;
    private JMenuItem           aHelpItem;
    private JMenuItem           aAboutItem;
    private JMenuItem           aLogItem;
    private JMenuItem           aSetupItem;
    private JMenuItem           aReloadItem;
    private JMenuItem           aQuitItem;
    private JMenuItem           aPlayItem;
    private JMenuItem           aStopItem;
    private JMenuItem           aCheckAllItem;
    private JMenuItem           aCheckNoneItem;
    private JMenuItem           aMoveDownItem;
    private JMenuItem           aMoveUpItem;


    /**
     * Create jTagger application.
     * Try to start from last used dir or a preset directory.
     */
    public JTagger() {
        super(Constants.APPNAME + " " + Constants.APPVERSION);
        Log.get().setLogLevel(3);

        JTAGGER = this;
        aMainWindow = new MainWindow();
        aDefaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
        aWaitCursor = new Cursor(Cursor.WAIT_CURSOR);
        AudioThread.get("Stop playing before deleting/updating any of the tracks!");
        NO_IMAGE = new ImageIcon(JTagger.loadImage("com/googlepages/dronten/jtagger/resource/NoImage.png"));
        setIcon("com/googlepages/dronten/jtagger/resource/jTagger.icon.png");

        new Pref(Constants.APPNAME);
        aMainWindow.create();
        createMenu();
        setContentPane(aMainWindow);
        windowRestore();

        String path = Pref.getPref(Constants.PREF_LASTDDIR, "");
        String path2 = Pref.getPref(Constants.PREF_STARTDIR, "");
        String path3 = Pref.getPref(Constants.PREF_STARTFROMLAST, "");

        if (path3.length() == 0 && path2.length() > 0) {
            aMainWindow.getLeftPanel().getDirTree().restore(path2);
            aMainWindow.getLeftPanel().setCurrentDirectory(path2);
            setMessage("Started from base directory", false);
        }
        else if (path.length() > 0) {
            aMainWindow.getLeftPanel().getDirTree().restore(path);
            aMainWindow.getLeftPanel().setCurrentDirectory(path);
            setMessage("Last used directory is restored", false);
        }
        else {
            setMessage("Select directory with audio tracks", false);
        }

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                quit();
            }
        });

        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    }


    /**
     * Respond to main menu.
     *
     * @param actionEvent Menu action event
     */
    public void actionPerformed(ActionEvent actionEvent) {
        if (actionEvent.getSource() == aReloadItem) {
            reloadDirectory();
        }
        else if (actionEvent.getSource() == aSetupItem) {
            String[] choices = Constants.OPTIONS_PREFERENCE;
            BaseSetup[] panels = {new GeneralSetup(), new MP3Setup()};
            SetupDialog dlg = new SetupDialog(this, "jTagger Setup", "&Close", choices, panels);

            dlg.setPreferredSize(new Dimension(500, 300));
            dlg.pack();
            dlg.centerOnApplication();
            dlg.setVisible(true);
            getMainWindow().getLeftPanel().reload();
        }
        else if (actionEvent.getSource() == aLogItem) {
            MessageDialog dlg = new MessageDialog(this, "jTagger Log", "&Close");

            dlg.setText(Log.get().getLogMessage());
            dlg.centerOnApplication();
            dlg.setVisible(true);
        }
        else if (actionEvent.getSource() == aCheckAllItem) {
            for (AudioTrack track : getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks()) {
                track.setSelected(true);
            }

            reloadAllTables();
        }
        else if (actionEvent.getSource() == aCheckNoneItem) {
            for (AudioTrack track : getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks()) {
                track.setSelected(false);
            }

            reloadAllTables();
        }
        else if (actionEvent.getSource() == aQuitItem) {
            quit();
        }
        else if (actionEvent.getSource() == aMoveDownItem) {
            getMainWindow().getTrackPane().getTrackTable().getTrackModel().moveDown(aSelectedRow);
            reloadAllTables();
        }
        else if (actionEvent.getSource() == aMoveUpItem) {
            getMainWindow().getTrackPane().getTrackTable().getTrackModel().moveUp(aSelectedRow);
            reloadAllTables();
        }
        else if (actionEvent.getSource() == aPlayItem) {
            AudioThread.get().setTrack(getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTrack(aSelectedRow));
        }
        else if (actionEvent.getSource() == aStopItem) {
            AudioThread.get().stopPlaying();
        }
        else if (actionEvent.getSource() == aHelpItem) {
            String help;
            help = "<html>";
            help += "<a name='top'><h2>jTagger " + Constants.APPVERSION + "</h2>";
            help += "Use jTagger on your own risk!.";

            help += "<h3>Usage</h3>";
            help += "Select a directory in the directory browser.<br>";
            help += "All valid MP3 tracks will appear in the track list.<br>";
            help += "Then you can edit names and MP3 tags for one or more files.<br>";
            help += "All options in the different option panels are parsed from top to bottom when jTagger does the name/tag update.<br>";
            help += "It's possible do use regular expressions in text remove operations.";

            help += "<h3>Images</h3>";
            help += "To use images you have to check the 'Use Images' check box.<br>";
            help += "If unchecked then all images will be removed during file update.<br>";
            help += "By default it is checked.<br>";
            help += "Only the first image inside a track can be set or changed.";

            help += "<h3>Updating</h3>";
            help += "Most views has an test preview, to see some of the batch changes that will be done.<br>";
            help += "If the filename or any tag is edited in the table, it will be saved immediately!";


            help += "<h3>Audio Files</h3>";
            help += "Edit filename, tags and image for a single tracks.<br>";
            help += "No changes are done until 'Write Tags' button has been pressed.";

            help += "<h3>Rename Files</h3>";
            help += "Do a batch rename of filenames.<br>";
            help += "No changes are done until 'Rename Files' button has been pressed.";

            help += "<h3>Rename Album Tags</h3>";
            help += "Do a batch rename of album tags.<br>";
            help += "Set also an image for all tracks.<br>";
            help += "No changes are done until 'Update Album Tags' button has been pressed.";

            help += "<h3>Rename Title Tags</h3>";
            help += "Do a batch rename of title tags.<br>";
            help += "No changes are done until 'Update Title Tags' button has been pressed.";
            help += "<br>";
            help += "<br>";
            help += "</html>";
            MessageDialog dlg = new MessageDialog(this, "jTagger Help", "&Close");

            dlg.setHtml(help);
            dlg.centerOnApplication();
            dlg.setVisible(true);
        }
        else if (actionEvent.getSource() == aAboutItem) {
            String help;
            help = "<html>";
            help += "<a name='top'><h2>jTagger " + Constants.APPVERSION + "</h2>Copright 2004 - 2009 by "  + Constants.EMAIL + "<br>";
            help += Constants.WWW + "<br>";
            help += "JTagger is distributed under the terms of the GPL v3 license.<br>";
            help += "<br>";
            help += "This program contains third party software:<br>";
            help += "JID3 mp3 tag library, LPGL2 License, Copyright 2004 - 2005 Paul Grebenc<br>";
            help += "JLayer mp3 playback, LPGL2 License, Copyright JavaZOOM 1999 - 2005<br>";
            help += "<br>";
            help += "Patches from:<br>";
            help += "Patrick Oosterveld<br>";
            help += "<br>";
            help += "</html>";
            MessageDialog dlg = new MessageDialog(this, "jTagger Help", "&Close");

            dlg.setHtml(help);
            dlg.centerOnApplication();
            dlg.setVisible(true);
        }
    }


    /**
     *
     */
    private void createMenu() {
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        JMenu tracksMenu = new JMenu("Tracks");
        JMenu helpMenu = new JMenu("Help");

        menuBar.add(fileMenu);
        menuBar.add(tracksMenu);
        menuBar.add(helpMenu);

        /*********************************************************************/

        aSetupItem = new JMenuItem("Setup...");
        aSetupItem.addActionListener(this);
        fileMenu.add(aSetupItem);

        aLogItem = new JMenuItem("Show Log...");
        aLogItem.addActionListener(this);
        fileMenu.add(aLogItem);

        aReloadItem = new JMenuItem("Reload Audio Tracks");
        aReloadItem.addActionListener(this);
        aReloadItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.Event.CTRL_MASK));
        fileMenu.add(aReloadItem);

        fileMenu.addSeparator();

        aQuitItem = new JMenuItem("Quit");
        aQuitItem.addActionListener(this);
        fileMenu.add(aQuitItem);

        /*********************************************************************/

        aCheckAllItem = new JMenuItem("Check All");
        aCheckAllItem.addActionListener(this);
        tracksMenu.add(aCheckAllItem);

        aCheckNoneItem = new JMenuItem("Check None");
        aCheckNoneItem.addActionListener(this);
        tracksMenu.add(aCheckNoneItem);

        tracksMenu.addSeparator();

        aMoveUpItem = new JMenuItem("Move Track Up");
        aMoveUpItem.addActionListener(this);
        aMoveUpItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.Event.CTRL_MASK));
        tracksMenu.add(aMoveUpItem);

        aMoveDownItem = new JMenuItem("Move Track Down");
        aMoveDownItem.addActionListener(this);
        aMoveDownItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.Event.CTRL_MASK));
        tracksMenu.add(aMoveDownItem);

        tracksMenu.addSeparator();

        aPlayItem = new JMenuItem("Play Selected Track");
        aPlayItem.addActionListener(this);
        tracksMenu.add(aPlayItem);

        aStopItem = new JMenuItem("Stop Playing Track");
        aStopItem.addActionListener(this);
        tracksMenu.add(aStopItem);

        /*********************************************************************/

        aHelpItem = new JMenuItem("Usage...");
        aHelpItem.addActionListener(this);
        helpMenu.add(aHelpItem);

        aAboutItem = new JMenuItem("About...");
        aAboutItem.addActionListener(this);
        helpMenu.add(aAboutItem);

        setJMenuBar(menuBar);
    }


    /**
     * Enable/disable menu items if a track is selected or not.
     *
     * @param enabled true to enable menu items
     */
    public void enableMenuItems(boolean enabled) {
        aCheckAllItem.setEnabled(enabled);
        aCheckNoneItem.setEnabled(enabled);
        aMoveUpItem.setEnabled(enabled);
        aMoveDownItem.setEnabled(enabled);
        aPlayItem.setEnabled(enabled);
        aStopItem.setEnabled(enabled);
    }


    /**
     * Get jTagger application object.
     *
     * @return return application object
     */
    public static JTagger get() {
        return JTAGGER;
    }


    /**
     * Get main window object.
     *
     * @return return main window object
     */
    public MainWindow getMainWindow() {
        return aMainWindow;
    }


    /**
     * Get codepage name.
     *
     * @return Code page
     */
    public String getCodePage() {
        return System.getProperty("file.encoding");
    }


    /**
     * Load image from resource.
     *
     * @param iconName Name of image
     * @return Image or null if failed
     */
    static public Image loadImage(String iconName) {
        Image icon = null;

        try {
            URL pathShell;
            ClassLoader cl = JTagger.class.getClassLoader();
            pathShell = cl.getResource(iconName);

            icon = Toolkit.getDefaultToolkit().getImage(pathShell);
        }
        catch (Exception e) {
            if (Constants.DEBUG) e.printStackTrace();
        }

        return icon;
    }


    /**
     * Quit application.
     * Save last used path and screen position.
     */
    public void quit() {
        try {
            AudioThread.get().stopPlaying();
            AudioThread.get().stopRunning();
        }
        catch (Exception e) {
            if (Constants.DEBUG) e.printStackTrace();
        }

        windowSave();
        setVisible(false);
        dispose();
        System.exit(0);
    }


    /**
     * Reload directory.
     */
    public void reloadDirectory() {
        getMainWindow().getLeftPanel().reload();
    }


    /**
     * Master data model TrackModel signals that data has changed.<br>
     * Let all data models depending on this update itself.<br>
     * Update also components in the panels.<br>
     */
    public void reloadAllTables() {
        getMainWindow().getTrackPane().getTrackTable().getTrackModel().fireTableDataChanged();
        getMainWindow().getFilePane().getFileTable().getFileModel().fireTableDataChanged();
        getMainWindow().getAlbumPane().getAlbumTable().getAlbumModel().fireTableDataChanged();
        getMainWindow().getTitlePane().getTitleTable().getTitleModel().fireTableDataChanged();

        getMainWindow().getFilePane().getFileTable().updateButtons();
        getMainWindow().getAlbumPane().getAlbumTable().updateButtons();
        getMainWindow().getTitlePane().getTitleTable().updateButtons();

        selectRow(aSelectedRow);
    }


    /**
     * Select row for all table widgets that displays filenames from the TrackModel object.
     *
     * @param row Row number in table
     */
    public void selectRow(int row) {
        aSelectedRow = row;

        if (aSelectedRow >= getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks().size()) {
            aSelectedRow = 0;
        }

        if (getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks().size() > 0) {
            aMainWindow.getTrackPane().getTrackTable().selectRow(aSelectedRow);
            aMainWindow.getFilePane().getFileTable().selectRow(aSelectedRow);
            aMainWindow.getAlbumPane().getAlbumTable().selectRow(aSelectedRow);
            aMainWindow.getTitlePane().getTitleTable().selectRow(aSelectedRow);
            enableMenuItems(true);
        }
        else {
            enableMenuItems(false);
        }
    }


    /**
     * Set code page.
     *
     * @param newCodePage New code page
     */
    public void setCodePage(String newCodePage) {
        if (newCodePage.length() > 0 || Pref.getPref(Constants.CODEPAGE, "").length() > 0) {
            Pref.setPref(Constants.CODEPAGE, newCodePage);
            Log.get().addTime(1, "Old codepage:" + System.getProperty("file.encoding"));

            Properties pi = System.getProperties();
            pi.put("file.encoding", newCodePage);
            System.setProperties(pi);

            Log.get().addTime(1, "New codepage:" + System.getProperty("file.encoding"));
        }
    }


    /**
     * Set application icon.
     *
     * @param iconName Name of resource image
     */
    public void setIcon(String iconName) {
        Image icon = loadImage(iconName);
        setIconImage(icon);
    }


    /**
     * Set statusbar message.
     *
     * @param message   Error message
     * @param error     true for an error message
     */
    public void setMessage(String message, boolean error) {
        if (error) {
            aMainWindow.getStatusBar().setErrorMessage(message);
        }
        else {
            aMainWindow.getStatusBar().setMessage(message);
        }
    }


    /**
     * Set normal cursor
     */
    public void setNormalCursor() {
        aMainWindow.setCursor(aDefaultCursor);
    }


    /**
     * Set application title text.
     *
     * @param additionalText Text after app name
     */
    public void setTitle(String additionalText) {
        if (additionalText != null) {
            super.setTitle(Constants.APPNAME + additionalText);
        }
        else {
            super.setTitle(Constants.APPNAME);
        }
    }


    /**
     * Set wait cursor
     */
    public void setWaitCursor() {
        aMainWindow.setCursor(aWaitCursor);
    }


    /**
     * Show error message.
     *
     * @param resName - Resource key
     */
    public void showErrorDialog(String resName) {
        JOptionPane.showMessageDialog(this, resName, Constants.APPNAME, JOptionPane.ERROR_MESSAGE);
    }


    /**
     * Show information message.
     *
     * @param resName - Resource key
     */
    public void showInfoDialog(String resName) {
        JOptionPane.showMessageDialog(this, resName, Constants.APPNAME, JOptionPane.INFORMATION_MESSAGE);
    }


    /**
     * Restore window size.
     */
    public void windowRestore() {
        double      w = Pref.getPref("width", 800.0);
        double      h = Pref.getPref("height", 600.0);
        double      x = Pref.getPref("xpos", 0.0);
        double      y = Pref.getPref("ypos", 0.0);
        boolean     max = Pref.getPref("screenmaximized", false);
        Dimension   screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        if (x > screenSize.getWidth()) {
            x = 0;
        }

        if (y > screenSize.getHeight()) {
            y = 0;
        }

        pack();
        setSize((int) w, (int) h);
        setLocation((int) x, (int) y);
        aMainWindow.getSplitPane().setDividerLocation((int) Pref.getPref("dirwidth", 200.0));
        aMainWindow.getTrackPane().setDividerLocation((int) Pref.getPref("trackwidth", 150.0));
        aMainWindow.getFilePane().setDividerLocation((int) Pref.getPref("filewidth", 150.0));
        aMainWindow.getAlbumPane().setDividerLocation((int) Pref.getPref("albumwidth", 150.0));
        aMainWindow.getTitlePane().setDividerLocation((int) Pref.getPref("titlewidth", 150.0));

        if (max) {
            setExtendedState(MAXIMIZED_BOTH);
        }
    }


    /**
     * Quit application.
     * Save last used path and screen position.
     */
    public void windowSave() {
        try {
            File file = new File(aMainWindow.getLeftPanel().getCurrentDirectory());

            if (file.exists() && file.isDirectory()) {
                Pref.setPref("path", aMainWindow.getLeftPanel().getCurrentDirectory());
            }
            else {
                Pref.setPref("path", "");
            }
        }
        catch (Exception e) {
            Pref.setPref("path", "");
        }

        try {
            Dimension size = getSize();
            Point pos = getLocation();

            Pref.setPref("width", Double.toString(size.getWidth()));
            Pref.setPref("height", Double.toString(size.getHeight()));
            Pref.setPref("xpos", Double.toString(pos.getX()));
            Pref.setPref("ypos", Double.toString(pos.getY()));
            Pref.setPref("dirwidth", Double.toString(aMainWindow.getSplitPane().getDividerLocation()));
            Pref.setPref("trackwidth", Double.toString(aMainWindow.getTrackPane().getDividerLocation()));
            Pref.setPref("filewidth", Double.toString(aMainWindow.getFilePane().getDividerLocation()));
            Pref.setPref("albumwidth", Double.toString(aMainWindow.getAlbumPane().getDividerLocation()));
            Pref.setPref("titlewidth", Double.toString(aMainWindow.getTitlePane().getDividerLocation()));

            if ((getExtendedState() & MAXIMIZED_BOTH) != 0) {
                Pref.setPref("screenmaximized", true);
            }
            else {
                Pref.setPref("screenmaximized", false);
            }
        }
        catch (Exception e) {
            if (Constants.DEBUG) e.printStackTrace();
        }
    }


    /**
     * Start app.<br>
     * It accepts look and feel class name.
     *
     * @param args Command arguments
     */
    public static void main(String[] args) {
        boolean failed = true;

        if (args.length > 0) {
            try {
                UIManager.setLookAndFeel(args[0]);
                failed = false;
            }
            catch (Exception e) {
                if (Constants.DEBUG) e.printStackTrace();
            }
        }

        if (failed) {
            try {
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
                failed = false;
            }
            catch (Exception e) {
                if (Constants.DEBUG) e.printStackTrace();
            }
        }

        if (failed) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
            catch (Exception e) {
                if (Constants.DEBUG) e.printStackTrace();
            }
        }

        try {
            final JTagger tagger = new JTagger();

            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    tagger.setVisible(true);
                }
            });
        }
        catch (Exception e) {
            System.out.println("Exception caught in main");
            System.out.println(e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
