/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.albumtab;

import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.model.AlbumModel;
import com.googlepages.dronten.jtagger.model.TrackModel;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;


/**
 * List data in a table that are synced to the TrackModel object.<br>
 *
 * @see TrackModel
 */
public class AlbumTablePanel extends JPanel {
    private static final long   serialVersionUID = 1;
    public JButton              aTestRenameButton = null;
    public JButton              aUndoTestButton = null;
    public JButton              aUpdateTagsButton = null;
    private JTable              aTable = null;
    private JPanel              aButtonPanel = null;


    /**
     * Create panel with a JTable grid showing some of the album data.
     */
    public AlbumTablePanel() {
        super();
        setLayout(new BorderLayout());
        aTable = new JTable(new AlbumModel());
        JScrollPane centerPane = new JScrollPane();
        JPanel southPanel = new JPanel();

        setBorder(BorderFactory.createEmptyBorder(5, 3, 5, 3));
        southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));

        aTable.getColumnModel().getColumn(0).setMinWidth(60);
        aTable.getColumnModel().getColumn(0).setPreferredWidth(60);
        aTable.getColumnModel().getColumn(0).setMaxWidth(100);

        aTable.getColumnModel().getColumn(1).setMinWidth(50);
        aTable.getColumnModel().getColumn(1).setPreferredWidth(50);
        aTable.getColumnModel().getColumn(1).setMaxWidth(100);

        aTable.getColumnModel().getColumn(2).setMinWidth(50);
        aTable.getColumnModel().getColumn(2).setPreferredWidth(200);
        aTable.getColumnModel().getColumn(2).setMaxWidth(1000);

        aTable.getColumnModel().getColumn(3).setMinWidth(100);
        aTable.getColumnModel().getColumn(3).setPreferredWidth(200);
        aTable.getColumnModel().getColumn(3).setMaxWidth(1000);

        aTable.getColumnModel().getColumn(4).setMinWidth(100);
        aTable.getColumnModel().getColumn(4).setPreferredWidth(200);
        aTable.getColumnModel().getColumn(4).setMaxWidth(1000);

        aTable.getColumnModel().getColumn(5).setMinWidth(75);
        aTable.getColumnModel().getColumn(5).setPreferredWidth(75);
        aTable.getColumnModel().getColumn(5).setMaxWidth(1000);

        aTable.setShowGrid(false);
        aTable.getTableHeader().setToolTipText("<html>Click table header to sort rows<br>ctrl-click to reverse sort order</html>");
        aTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        aTable.getTableHeader().addMouseListener(getAlbumModel().aMouseHandler);
        aTable.setToolTipText("Double click to edit track, title, artist or album");

        centerPane.getViewport().setView(aTable);
        southPanel.add(Box.createRigidArea(new Dimension(0, 3)));
        aButtonPanel = ComponentFactory.createPanelWithTwoColumns(1, 3);

        aTestRenameButton = ComponentFactory.createButton("&Test Rename Album", "", getAlbumModel());
        aButtonPanel.add(aTestRenameButton);

        aUndoTestButton = ComponentFactory.createButton("&Undo Test", "", getAlbumModel());
        aButtonPanel.add(aUndoTestButton);

        aUpdateTagsButton = ComponentFactory.createButton("Write Album Tags", "", getAlbumModel());
        aButtonPanel.add(aUpdateTagsButton);

        southPanel.add(aButtonPanel);
        add(centerPane, BorderLayout.CENTER);
        add(southPanel, BorderLayout.SOUTH);

        aTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }

                ListSelectionModel lsm = (ListSelectionModel) e.getSource();

                if (lsm.isSelectionEmpty() == false) {
                    int selectedRow = lsm.getMinSelectionIndex();

                    JTagger.get().selectRow(selectedRow);
                }
            }
        });
    }


    /**
     * Get the AlbumModel object.
     *
     * @return return AlbumModel object
     */
    public AlbumModel getAlbumModel() {
        return (AlbumModel) aTable.getModel();
    }


    /**
     * Select row on grid.
     *
     * @param row
     */
    public void selectRow(int row) {
        if (row >= 0) {
            ListSelectionModel lsm = aTable.getSelectionModel();
            lsm.setSelectionInterval(row, row);
        }
    }


    /**
     * Enable/disable buttons according to table status
     */
    public void updateButtons() {
        Component[] components = aButtonPanel.getComponents();
        for (Component component : components) {
            if (JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks().size() > 0) {
                component.setEnabled(true);
            }
            else {
                component.setEnabled(false);
            }
        }
    }
}
