/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.renamer;

import com.googlepages.dronten.jtagger.audio.AudioTrack;


/**
 * The file renamer object. Set the public members and call rename.
 */
public class FileRenamer extends BaseRenamer {
    public String   aUseTitle = null;
    public String   aUseText = null;
    public String   aRemoveIllegal = null;
    public String   aRemoveLeading = null;
    public String   aRemoveTrailing = null;
    public String   aRemoveText = null;
    public String   aReplaceText = null;
    public String   aUseAlbum = null;
    public String   aUseArtist = null;
    public String   aInsertText = null;
    public String   aAppendText = null;
    public int      aNumber = 0;
    public String   aStartNumber = null;
    public String   aNumberSize = null;
    public String   aNumberSeparator = null;
    public int      aCapitalize = 0;
    public int      aExtension = 0;


    /**
     *
     */
    public FileRenamer() {
        super(true);
    }


    /**
     * Create new filename.
     *
     * @param track         Audio track
     * @throws Exception    Throws exception if error found
     */
    @Override public void rename(AudioTrack track) throws Exception {
        if (aUseTitle != null) {
            track.aFileName = setString(track.aFileName, track.aTitle);
        }

        if (aUseText != null) {
            track.aFileName = setString(track.aFileName, aUseText);
        }

        if (aRemoveLeading != null) {
            track.aFileName = removeLeading(track.aFileName);
        }

        if (aRemoveTrailing != null) {
            track.aFileName = removeTrailing(track.aFileName);
        }

        if (aRemoveText != null && aReplaceText != null) {
            track.aFileName = replace(track.aFileName, aRemoveText, aReplaceText);
        }
        else if (aRemoveText != null) {
            track.aFileName = replace(track.aFileName, aRemoveText, "");
        }

        if (aUseAlbum != null) {
            track.aFileName = track.aAlbum + aUseAlbum + track.aFileName;
        }

        if (aUseArtist != null) {
            track.aFileName = track.aArtist + aUseArtist + track.aFileName;
        }

        if (aInsertText != null) {
            track.aFileName = aInsertText + track.aFileName;
        }

        if (aAppendText != null) {
            track.aFileName = appendText(track.aFileName, aAppendText);
        }

        if (aNumber == 1) {
            track.aFileName = insertNumber(track.aFileName, track.aTrack, aNumberSize, aNumberSeparator, false);
        }
        else if (aNumber == 2) {
            track.aFileName = insertNumber(track.aFileName, aStartNumber, aNumberSize, aNumberSeparator, true);
        }
        else if (aNumber == 3) {
            track.aFileName = appendNumber(track.aFileName, track.aTrack, aNumberSize, aNumberSeparator, false);
        }
        else if (aNumber == 4) {
            track.aFileName = appendNumber(track.aFileName, aStartNumber, aNumberSize, aNumberSeparator, true);
        }

        if (aRemoveIllegal != null) {
            track.aFileName = track.aFileName.replaceAll("[/:?\"]", "");
        }

        if (aCapitalize > 0) {
            track.aFileName = capitalize(track.aFileName, aCapitalize);
        }

        if (aExtension > 0) {
            track.aFileName = capitalizeExtension(track.aFileName, aExtension);
        }

        aTrackCount++;
    }
}
