/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.renamer;

import com.googlepages.dronten.jtagger.audio.AudioTrack;


/**
 * The file renamer object.<br>
 * Set the public members and call rename.
 */
public class TitleRenamer extends BaseRenamer {
    public String   aUseFileName = null;
    public String   aUseTitle = null;
    public String   aRemoveLeading = null;
    public String   aRemoveTrailing = null;
    public String   aRemoveText = null;
    public String   aReplaceText = null;
    public String   aInsertText = null;
    public String   aAppendText = null;
    public int      aNumber = 0;
    public String   aStartNumber = null;
    public String   aNumberSize = null;
    public String   aNumberSeparator = null;
    public int      aCapitalize = 0;


    /**
     *
     */
    public TitleRenamer() {
        super(false);
    }


    /**
     * Set and update mp3 tags.
     *
     * @param track         Audio track
     * @throws Exception    Throws exception if error found
     */
    @Override public void rename(AudioTrack track) throws Exception {
        if (aUseFileName != null) {
            aSplitString = true;
            split(track.aFileName);
            track.aTitle = aPart1;
            aSplitString = false;
        }

        if (aUseTitle != null) {
            track.aTitle = aUseTitle;
        }

        if (aRemoveLeading != null) {
            track.aTitle = removeLeading(track.aTitle);
        }

        if (aRemoveTrailing != null) {
            track.aTitle = removeTrailing(track.aTitle);
        }

        if (aRemoveText != null && aReplaceText != null) {
            track.aTitle = replace(track.aTitle, aRemoveText, aReplaceText);
        }
        else if (aRemoveText != null) {
            track.aTitle = replace(track.aTitle, aRemoveText, "");
        }

        if (aInsertText != null) {
            track.aTitle = aInsertText + track.aTitle;
        }

        if (aAppendText != null) {
            track.aTitle += aAppendText;
        }

        if (aNumber == 1) {
            track.aTitle = insertNumber(track.aTitle, track.aTrack, aNumberSize, aNumberSeparator, false);
        }
        else if (aNumber == 2) {
            track.aTitle = insertNumber(track.aTitle, aStartNumber, aNumberSize, aNumberSeparator, true);
        }
        else if (aNumber == 3) {
            track.aTitle = appendNumber(track.aTitle, track.aTrack, aNumberSize, aNumberSeparator, false);
        }
        else if (aNumber == 4) {
            track.aTitle = appendNumber(track.aTitle, aStartNumber, aNumberSize, aNumberSeparator, true);
        }

        if (aCapitalize > 0) {
            track.aTitle = capitalize(track.aTitle, aCapitalize);
        }

        aTrackCount++;
    }
}
