/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.thread;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.renamer.BaseRenamer;
import com.googlepages.dronten.jtagger.renamer.FileRenamer;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Progress;


/**
 * FileThread renames files in it's own thread.
 */
public class BatchWorkThread extends BaseThread {
    private BaseRenamer aRenamer;


    /**
     * @param log
     * @param progress
     * @param renamer
     */
    public BatchWorkThread(Log log, Progress progress, BaseRenamer renamer) {
        super(log, progress);

        long count = 0;
        for (AudioTrack track : JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks()) {
            if (track.isSelected()) {
                count++;
            }
        }
        aRenamer = renamer;
        Progress.get().clear();
        aProgress.appendTask(count, "");
    }


    /**
     *
     */
    public void run() {
        int progressCount = 0;
        JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().clearTest();

        try {
            for (AudioTrack track : JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks()) {
                AudioTrack track2 = (AudioTrack) track.clone();

                if (!aRunning) {
                    break;
                }

                if (track.isSelected()) {
                    progressCount++;
                    aProgress.setMinorProgress(progressCount);
                    aProgress.replaceMinorMessage(track.aFullFileName);
                    Thread.sleep(Constants.SLEEP);

                    aRenamer.rename(track);

                    if (aRenamer instanceof FileRenamer) {
                        Log.get().addTime(2, String.format("Changing filename for %s to %s", track2.aFileName, track.aFileName));
                        track2.rename(track.aFileName);
                        track.aFullFileName = track2.aFullFileName;
                    }
                    else {
                        Log.get().addTime(2, String.format("Changing tags for %s (%s)", track.aFileName, track.aTitle));
                        track.write(AudioTrack.getWriteTrackType(track.aFileName));
                    }
                }
            }

            JTagger.get().setMessage(Constants.MESSAGE_WRITE_OK, false);
        }
        catch (Exception e) {
            Log.get().addTime(1, e.getMessage());
            JTagger.get().setMessage(e.getMessage(), true);
        }
        finally {
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().clearTest();
            close();
        }
    }
}
