/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.thread;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.audio.AudioTrackComparator;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Progress;

import java.io.File;


/**
 * DirectoryReaderThread scans a directory for audio files.
 */
public class DirectoryReaderThread extends BaseThread {
    private File[] aFiles = null;


    /**
     * @param log
     * @param progress
     * @param currDir
     */
    public DirectoryReaderThread(Log log, Progress progress, String currDir) {
        super(log, progress);

        File dir = new File(currDir);
        aFiles = dir.listFiles();

        if (aFiles != null && aFiles.length > 0) {
            aProgress.appendTask((long) aFiles.length, currDir);
            Log.get().addTime(2, "Reading directory " + currDir);
        }
    }


    /**
     *
     */
    public void run() {
        try {
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().clear();
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().clearTest();

            if (aFiles != null) {
                int count = 0;

                for (File f : aFiles) {
                    if (!aRunning) {
                        break;
                    }

                    count++;
                    aProgress.setMinorProgress(count);
                    Thread.sleep(Constants.SLEEP);

                    if (f.isFile() && f.getName().toLowerCase().endsWith(".mp3")) {
                        try {
                            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().add(new AudioTrack(f.getName(), f.getPath()));
                        }
                        catch (Exception e) {
                            Log.get().addTime(1, "Exception for: " + f.getName());
                            Log.get().addTime(1, e.getMessage());
                        }
                    }
                }
            }
        }
        catch (Exception x) {
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks().clear();
            Log.get().addTime(1, x.getMessage());
        }
        finally {
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().setSortAscending(true);
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getComparator().aSortType = AudioTrackComparator.SORT.COMPARE_FILE;
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().sort();
            JTagger.get().selectRow(0);
            close();
        }
    }
}
